import { Channel } from './Channel';
import { WebGroupState } from './index.common.doc';
import { log } from './misc/util';
import { webChannelDefaultOptions } from './WebChannel';
import { wcs, WebGroup } from './WebChannelFacade';
import { WebSocketBuilder } from './WebSocketBuilder';
var urlLib = require('url');
var uws = require('uws');
var botDefaultOptions = {
    url: '',
    perMessageDeflate: false,
    leaveOnceAlone: true,
    server: undefined,
    webGroupOptions: webChannelDefaultOptions,
};
var Bot = (function () {
    function Bot(options) {
        this.wcOptions = Object.assign({}, webChannelDefaultOptions, options.webGroupOptions);
        var fullOptions = Object.assign({}, botDefaultOptions, options);
        fullOptions.webGroupOptions = this.wcOptions;
        this.leaveOnceAlone = fullOptions.leaveOnceAlone;
        this.server = fullOptions.server;
        this.listenUrl = fullOptions.url;
        this.perMessageDeflate = fullOptions.perMessageDeflate;
        this.webGroups = new Map();
        this.onWebGroup = function none() { };
        this.onError = function none() { };
        this.init();
    }
    Object.defineProperty(Bot.prototype, "url", {
        get: function () {
            if (this.listenUrl !== '') {
                return this.listenUrl;
            }
            else {
                var info = this.server.address();
                return "ws://" + info.address + ":" + info.port;
            }
        },
        enumerable: true,
        configurable: true
    });
    Bot.prototype.init = function () {
        var _this = this;
        this.webSocketServer = new uws.Server({
            perMessageDeflate: this.perMessageDeflate,
            verifyClient: function (info) { return _this.validateURLQuery(info); },
            server: this.server,
        });
        var serverListening = this.server || this.webSocketServer;
        serverListening.on('listening', function () { return WebSocketBuilder.listenUrl.next(_this.url); });
        this.webSocketServer.on('error', function (err) {
            WebSocketBuilder.listenUrl.next('');
            _this.onError(err);
        });
        this.webSocketServer.on('connection', function (ws) {
            var _a = _this.readURLQuery(ws.upgradeReq.url), type = _a.type, wcId = _a.wcId, senderId = _a.senderId, key = _a.key;
            var webSocketBuilder;
            if (type === Channel.WITH_MEMBER) {
                var wg = new WebGroup(_this.wcOptions);
                _this.webGroups.set(wcId, wg);
                var wc_1 = wcs.get(wg);
                if (_this.leaveOnceAlone) {
                    wc_1.onAlone = function () {
                        wc_1.leave();
                        _this.webGroups.delete(wcId);
                    };
                }
                wc_1.init(key, wcId);
                _this.onWebGroup(wg);
                webSocketBuilder = wc_1.webSocketBuilder;
            }
            else {
                var wg = _this.webGroups.get(wcId);
                var wc = wcs.get(wg);
                webSocketBuilder = wc.webSocketBuilder;
            }
            webSocketBuilder.newWebSocket(ws, senderId, type);
        });
    };
    Bot.prototype.validateURLQuery = function (info) {
        try {
            var _a = this.readURLQuery(info.req.url), type = _a.type, wcId = _a.wcId, key = _a.key;
            switch (type) {
                case Channel.WITH_INTERNAL:
                    return this.webGroups.has(wcId);
                case Channel.WITH_MEMBER: {
                    var wg = this.webGroups.get(wcId);
                    return !!key && (wg === undefined || wg.state === WebGroupState.LEFT);
                }
                case Channel.WITH_JOINING: {
                    var wg = this.webGroups.get(wcId);
                    return !!key && wg !== undefined && wg.key === key && wg.state !== WebGroupState.LEFT;
                }
                default:
                    return false;
            }
        }
        catch (err) {
            log.warn(err.message);
            return false;
        }
    };
    Bot.prototype.readURLQuery = function (url) {
        var _a = urlLib.parse(url, true).query, type = _a.type, wcId = _a.wcId, senderId = _a.senderId, key = _a.key;
        if (!type) {
            throw new Error('Query parse error: "type" parameter is undefined');
        }
        if (!wcId) {
            throw new Error('Query parse error: "wcId" parameter is undefined');
        }
        if (!senderId) {
            throw new Error('Query parse error: "senderId" parameter is undefined');
        }
        return {
            type: Number.parseInt(type, 10),
            wcId: Number.parseInt(wcId, 10),
            senderId: Number.parseInt(senderId, 10),
            key: key,
        };
    };
    return Bot;
}());
export { Bot };
//# sourceMappingURL=Bot.js.map