import { env } from './env';
export var MIN_ID = 2147483648;
export var isBrowser = typeof window === 'undefined' ? false : true;
export function isOnline() {
    return isBrowser ? navigator.onLine : true;
}
export function isVisible() {
    return isBrowser ? document.visibilityState === 'visible' : true;
}
export function validateWebSocketURL(url) {
    var regex = /^(wss|ws):\/\/((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])|(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9]))(:[0-9]{1,5})?(\/.*)?$/;
    if (!new RegExp(regex, 'i').test(url)) {
        throw new Error("Invalid URL: " + url);
    }
}
export function generateKey() {
    var mask = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    var length = 42;
    var values = randNumbers(length);
    var result = '';
    for (var i = 0; i < length; i++) {
        result += mask[values[i] % mask.length];
    }
    return result;
}
export function generateId(exclude) {
    if (exclude === void 0) { exclude = []; }
    var id = randNumbers()[0];
    if (id < MIN_ID) {
        id += MIN_ID;
    }
    if (exclude.includes(id)) {
        return generateId(exclude);
    }
    return id;
}
var MAX_KEY_LENGTH = 512;
export function validateKey(key) {
    if (typeof key !== 'string') {
        throw new Error("The key type \"" + typeof key + "\" is not a \"string\"");
    }
    else if (key === '') {
        throw new Error('The key is an empty string');
    }
    else if (key.length > MAX_KEY_LENGTH) {
        throw new Error("The key length of " + key.length + " exceeds the maximum of " + MAX_KEY_LENGTH + " characters");
    }
    return true;
}
export function extractHostnameAndPort(url) {
    return url.split('/')[2];
}
function randNumbers(length) {
    if (length === void 0) { length = 1; }
    var res;
    if (isBrowser) {
        res = new Uint32Array(length);
        env.crypto.getRandomValues(res);
    }
    else {
        res = [];
        var bytes = env.cryptoNode.randomBytes(4 * length);
        for (var i = 0; i < bytes.length; i += 4) {
            res[res.length] = bytes.readUInt32BE(i, true);
        }
    }
    return res;
}
export function equal(array1, array2) {
    return (array1 !== undefined &&
        array2 !== undefined &&
        array1.length === array2.length &&
        array1.every(function (v) { return array2.includes(v); }));
}
export function isWebSocketSupported() {
    return !!env.WebSocket;
}
export function isWebRTCSupported() {
    return !!env.RTCPeerConnection && 'createDataChannel' in env.RTCPeerConnection.prototype;
}
export * from './util.log';
//# sourceMappingURL=util.js.map