import { log } from '../../misc/util';
import { ConnectionError } from './ConnectionError';
var ConnectionsInProgress = (function () {
    function ConnectionsInProgress() {
        this.connections = new Map();
    }
    ConnectionsInProgress.prototype.create = function (id, connectionTimeout, responseTimeout, onConnectionTimeoutCallback) {
        var _this = this;
        var connection = {};
        connection.promise = new Promise(function (resolveResponse, rejectResponse) {
            var responseTimer = setTimeout(function () { return connection.reject(new Error(ConnectionError.RESPONSE_TIMEOUT)); }, responseTimeout);
            connection.resolve = function () {
                clearTimeout(responseTimer);
                resolveResponse();
                connection.promise = new Promise(function (resolveConnection, rejectConnection) {
                    var connectionTimer = setTimeout(function () {
                        log.channelBuilder('on Connection timeout callback');
                        onConnectionTimeoutCallback();
                        connection.reject(new Error(ConnectionError.CONNECTION_TIMEOUT));
                    }, connectionTimeout);
                    connection.resolve = function () {
                        clearTimeout(connectionTimer);
                        _this.connections.delete(id);
                        resolveConnection();
                    };
                    connection.reject = function (err) {
                        connection.promise.catch(function () { });
                        clearTimeout(connectionTimer);
                        _this.connections.delete(id);
                        rejectConnection(err);
                    };
                });
            };
            connection.reject = function (err) {
                connection.promise.catch(function () { });
                clearTimeout(responseTimer);
                _this.connections.delete(id);
                rejectResponse(err);
            };
            _this.connections.set(id, connection);
        });
        return connection;
    };
    ConnectionsInProgress.prototype.get = function (id) {
        return this.connections.get(id);
    };
    ConnectionsInProgress.prototype.has = function (streamId, id) {
        return this.connections.has(id);
    };
    ConnectionsInProgress.prototype.clean = function () {
        this.connections.forEach(function (c) { return c.reject(new Error(ConnectionError.CLEAN)); });
        this.connections.clear();
    };
    return ConnectionsInProgress;
}());
export { ConnectionsInProgress };
//# sourceMappingURL=ConnectionsInProgress.js.map